import {Circle, lerp, Point, Rect} from "./engine/datatypes.ts";

export function displace(
  bbox: Circle,
  dxy: Point,
  getContact: (where: Circle) => Point | null,
  options?: { bounce?: number },
): { bbox: Circle; displacement: Point; dxy: Point } {
  let nSteps = 40;
  let nRedirections = 40;
  let bounce = options?.bounce ?? 0;

  let xy = bbox.center;
  let redirections = 0;
  for (let i = 0; i < nSteps; i++) {
    let trialXy = xy.offset(new Point(dxy.x / nSteps, dxy.y / nSteps));
    let trialBbox = new Circle(trialXy, bbox.radius);

    let contact = getContact(trialBbox);
    if (contact) {
      let normal = contact.offset(trialXy.negate());
      let mag = normal.distance(new Point(0, 0));
      let nx = mag == 0 ? 0 : normal.x / mag;
      let ny = mag == 0 ? 0 : normal.y / mag;

      let dot = dxy.x * nx + dxy.y * ny;
      if (redirections < nRedirections) {
        dxy = new Point(dxy.x - lerp(bounce, 1, 2) * dot * nx, dxy.y - lerp(bounce, 1, 2) * dot * ny);
        i -= 1; // try again with reflection
        redirections += 1;
      } else {
        dxy = new Point(0, 0);
        break;
      }
    } else {
      xy = trialXy;
    }
  }

  return {
    bbox: new Circle(xy, bbox.radius),
    displacement: xy.offset(bbox.center.negate()),
    dxy,
  };
}
